-- Register as ACE2 Addon
Avion_Core = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceModuleCore-2.0", "FuBarPlugin-2.0")
Avion_Core:SetModuleMixins("AceEvent-2.0")


-- AvionAddin-1.0 to show those neat animations
local AvionAddin = AceLibrary("AvionAddin-1.0")


-- Translation
local L = AceLibrary("AceLocale-2.2"):new("Avion_Core")


-- Register SavedVariables
Avion_Core:RegisterDB("AvionDB")
Avion_Core:RegisterDefaults("profile", { 
	settings = {
	},
	version = 2.0
})


-- This methods inherits every module, makes them say hello to core
function Avion_Core.modulePrototype:OnInitialize()
	-- Register DB for saving
	self:RegisterModuleDB()
	
	-- Listen to events
	self:UpdateEventRegistration(false)
	
	-- Add optiontable
	Avion_Core:UpdateModuleOptionTable(self)
end


-- Register frame cache
function Avion_Core:OnEnable()
	-- Register AvionAddin DB
	AvionAddin:RegisterAnimationDB("AvionDB")
	
	-- If any Module available, register its events
	for name, module in self:IterateModules() do
		module:UpdateEventRegistration(false)
	end
	
	-- Register events that cause table to get updated
	self:RegisterEvent("AvionAddin_MenuUpdate", "UpdateAllModulesOptionTable")
	
	-- Update optiontable once	
	self:UpdateOptionTable()
	
	-- Register when animation gets deleted, so eventIDLists may be checked
	self:RegisterEvent("AvionAddin_DeleteAnimation", "AnimationDeleted")
end


-- Delete and close everything
function Avion_Core:OnDisable()
	-- Stop all animations
	AvionAddin:UnregisterAllAnimations()
	
	-- Unregister all events
	for name, module in self:IterateModules() do
		module:UpdateEventRegistration(true)
	end
	
	-- Unregister all events
	self:UnregisterEvent("AvionAddin_MenuUpdate")
	self:UnregisterEvent("AvionAddin_DeleteAnimation")
end


-- Build optiontable to select animation from
function Avion_Core:BuildAnimationList(tableVar)
	local result = {}
	
	-- Add toogle button for each animation
	for id, name in AvionAddin:AnimationIterator() do
		result[id] =  {
			type = 'toggle',
			name = name,
			desc = L["Animation_Desc"],
			get = function()
				return (tableVar[id] == true)
			end,
			set = function(toggle)
				if (toggle == true) then
					tableVar[id] = true
				else
					tableVar[id] = nil
				end
			end,
		}
	end
	
	return result
end


-- Update modules optionTable
function Avion_Core:UpdateModuleOptionTable(module)
	-- Insert new/updated options
	self.OnMenuRequest.args[module.name] = module:UpdateOptionTable()
end


-- Update all modules
function Avion_Core:UpdateAllModulesOptionTable()
	for name, module in self:IterateModules() do
		self:UpdateModuleOptionTable(module)
	end
end


-- Update complete optiontable
function Avion_Core:UpdateOptionTable()
	-- AvionAddin options (manage list of animations)
	self.OnMenuRequest.args.core = AvionAddin:UpdateOptionTable()
	self.OnMenuRequest.args.core.name = L["Avion_Animations"]

	-- Update module options
	for name, module in self:IterateModules() do
		self:UpdateModuleOptionTable(module)
	end
end


-- Trigger MANY animations at once
function Avion_Core:TriggerAnimationsByID(triggerID, animationIDList)
	local animationTable
	
	-- Trigger each animation
	for animationID, _ in pairs(animationIDList) do
		AvionAddin:TriggerAnimationByID(string.format("%s%d", triggerID, animationID), animationID)
	end
end


-- Animation was deleted, make sure no module is using it!
function Avion_Core:AnimationDeleted(animationID)
	self:CallMethodOnAllModules("AnimationDeleted" , animationID)
end


-- Some FuBar stuff
Avion_Core.name                   	= L["AddonName"]
Avion_Core.title                   	= L["AddonName"]
Avion_Core.hasIcon                	= true
Avion_Core.defaultMinimapPosition 	= 200
Avion_Core.cannotDetachTooltip    	= true
Avion_Core.independentProfile     	= false
Avion_Core.defaultPosition        	= "RIGHT"
Avion_Core.hideWithoutStandby     	= true
Avion_Core.hasNoColor				= true
Avion_Core.OnMenuRequest			= {
	order = 1,
	type = 'group',
	name = L["AvionSettings"],
	desc = L["AvionSettings_Desc"],
	args = { }
}
