-- Create new Module
local Avion_Buffs 	= Avion_Core:NewModule("Avion_Buffs")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Buffs")
local BC = AceLibrary("Babble-Class-2.2")


-- Module information, default options and name
Avion_Buffs.name 	= "Avion_Buffs"


-- Register SavedVariables
function Avion_Buffs:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Buffs", "profile", { 
		BuffGained = {
			player = {
			},
			raid = {
			},
			target = {
			},
			focus = {
			},
			class = {
			},
			name = {
			},
		},
		BuffLost = {
			player = {
			},
			raid = {
			},
			target = {
			},
			focus = {
			},
			class = {
			},
			name = {
			},
		},
		DebuffGained = {
			player = {
			},
			raid = {
			},
			target = {
			},
			focus = {
			},
			class = {
			},
			name = {
			},
		},
		DebuffLost = {
			player = {
			},
			raid = {
			},
			target = {
			},
			focus = {
			},
			class = {
			},
			name = {
			},
		},
		DebuffTypeGained = {
			player = {
			},
			raid = {
			},
			target = {
			},
			focus = {
			},
			class = {
			},
			name = {
			},
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Buffs")
end


-- (Un)register (un)needed events
function Avion_Buffs:UpdateEventRegistration(unLoad)
	self:UpdateEventRegistration_Player(unLoad)
	self:UpdateEventRegistration_Unit(unLoad)
end


-- (Un)register (un)needed events
function Avion_Buffs:UpdateEventRegistration_Player(unLoad)
	local isEnabled

	-- Test if any gained buff is enabled
	isEnabled = false
	for _, data in pairs(self.db.profile.BuffGained.player) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
	
	-- (Un)register events
	if (not unLoad and isEnabled) then
		self:RegisterEvent("SpecialEvents_PlayerBuffGained")
	else
		if (self:IsEventRegistered("SpecialEvents_PlayerBuffGained")) then
			self:UnregisterEvent("SpecialEvents_PlayerBuffGained")
		end
	end
	
	-- Test if any lost buff is enabled
	for _, data in pairs(self.db.profile.BuffLost.player) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
		
	-- (Un)register events
	if (not unLoad and isEnabled) then
		self:RegisterEvent("SpecialEvents_PlayerBuffLost")
	else
		if (self:IsEventRegistered("SpecialEvents_PlayerBuffLost")) then
			self:UnregisterEvent("SpecialEvents_PlayerBuffLost")
		end
	end
	
	-- Test if any lost buff is enabled
	for _, data in pairs(self.db.profile.DebuffLost.player) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
		
	-- (Un)register events
	if (not unLoad and isEnabled) then
		self:RegisterEvent("SpecialEvents_PlayerDebuffLost")
	else
		if (self:IsEventRegistered("SpecialEvents_PlayerDebuffLost")) then
			self:UnregisterEvent("SpecialEvents_PlayerDebuffLost")
		end
	end
	
	-- Test if any debuff is enabled
	isEnabled = false
	for _, data in pairs(self.db.profile.DebuffGained.player) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
	if (not isEnabled) then
		for _, data in pairs(self.db.profile.DebuffTypeGained.player) do
			if (data.enabled) then
				isEnabled = true
				break
			end
		end
	end
	
	-- (Un)register events
	if (not unLoad and isEnabled) then		
		self:RegisterEvent("SpecialEvents_PlayerDebuffGained")
	else
		if (self:IsEventRegistered("SpecialEvents_PlayerDebuffGained")) then
			self:UnregisterEvent("SpecialEvents_PlayerDebuffGained")
		end
	end
end


-- (Un)register (un)needed events
function Avion_Buffs:UpdateEventRegistration_Unit(unLoad)
	local isEnabled = {
	} 
	
	-- No need to check, when disabling
	if (not unLoad) then
		local data = self.db.profile

		-- Test if ANY entry is enabled
		for dataType, dataTable in pairs (data) do
			isEnabled[dataType] = false
		
			for _, unitIDs in pairs(dataTable) do
				for unitID, data in pairs(unitIDs) do
					if (unitID == "class" or unitID == "name") then
						for _, data_ in pairs(data) do
							if (data_[dataType]) then
								isEnabled[dataType] = true
								break
							end
						end
					else
						if (data.enabled) then
							isEnabled[dataType] = true
							break
						end
					end
					
					if (isEnabled[dataType]) then
						break
					end
				end
				
				if (isEnabled[dataType]) then
					break
				end
			end
		end
	end

	-- (Un)register events
	if (isEnabled["BuffGained"]) then
		self:RegisterEvent("SpecialEvents_UnitBuffGained")
	else
		if (self:IsEventRegistered("SpecialEvents_UnitBuffGained")) then
			self:UnregisterEvent("SpecialEvents_UnitBuffGained")
		end
	end
	
	-- (Un)register events
	if (isEnabled["BuffLost"]) then
		self:RegisterEvent("SpecialEvents_UnitBuffLost")
	else
		if (self:IsEventRegistered("SpecialEvents_UnitBuffLost")) then
			self:UnregisterEvent("SpecialEvents_UnitBuffLost")
		end
	end
	
	-- (Un)register events
	if (isEnabled["DebuffLost"]) then
		self:RegisterEvent("SpecialEvents_UnitDebuffLost")
	else
		if (self:IsEventRegistered("SpecialEvents_UnitDebuffLost")) then
			self:UnregisterEvent("SpecialEvents_UnitDebuffLost")
		end
	end
	
	-- (Un)register events
	if (isEnabled["DebuffGained"] or isEnabled["DebuffTypeGained"]) then		
		self:RegisterEvent("SpecialEvents_UnitDebuffGained")
	else
		if (self:IsEventRegistered("SpecialEvents_UnitDebuffGained")) then
			self:UnregisterEvent("SpecialEvents_UnitDebuffGained")
		end
	end
end


-- Delete trigger from DB
function Avion_Buffs:NewDeBuff(dataBaseVar, itsName, unitID)
	-- Add to DB
	if (unitID == "name" or unitID == "class") then
		dataBaseVar[itsName] =  { }
	else
		dataBaseVar[itsName] =  {
			animationIDList = { },
			enabled = true,
		}
	end
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Delete trigger from DB
function Avion_Buffs:DeleteDeBuff(dataBaseVar, itsName)
	-- Delete from DB
	dataBaseVar[itsName] = nil
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Build optiontable to insert new DeBuff into DB
function Avion_Buffs:BuildNewDeBuffOptionTable(dataBaseVar, mode, unitID)
	if (mode == "DebuffTypeGained") then
		local result = {}
		for _, dType in pairs({"Magic", "Curse", "Disease", "Poison"}) do
			if (not dataBaseVar[dType]) then
				result[dType] = {
					type = 'execute',
					name = L[dType],
					desc = L["Type_Desc"],
					func  = function(toggle)
						self:NewDeBuff(dataBaseVar, dType, unitID)
					end,
				}
			end
		end
		
		return {
			type = 'group',
			order = 1,
			name = L["New"],
			desc = L["New_Desc"],
			args = result
		}
	else
		return {
			type = 'text',
			order = 1,
			name = L["New"],
			desc = L["New_Desc"],
			usage = L["New_Usage"],
			get = function()
				return ""
			end,
			set = function(input)
				self:NewDeBuff(dataBaseVar, input, unitID)
			end,
			validate = function(input)
				return (not dataBaseVar[input] and strlen(input) > 0)
			end
		}
	end
end	


function Avion_Buffs:BuildNewDeBuffOptionTable_Special(dataBaseVar, unitID)
	if (unitID == "class") then
		local result = {}
		for _, class in pairs({"Druid", "Hunter", "Mage", "Paladin", "Priest", "Rogue", "Shaman", "Warlock", "Warrior"}) do
			if (not dataBaseVar[class]) then
				result[class] = {
					type = 'execute',
					name = BC[class],
					desc = L["Class_Desc"],
					func  = function(toggle)
						self:NewDeBuff(dataBaseVar, class)
					end,
				}
			end
		end
		
		return {
			type = 'group',
			order = 1,
			name = L["New"],
			desc = L["New_Desc"],
			args = result
		}
	else
		return {
			type = 'text',
			order = 1,
			name = L["New"],
			desc = L["New_Desc"],
			usage = L["New_Usage"],
			get = function()
				return ""
			end,
			set = function(input)
				self:NewDeBuff(dataBaseVar, input)
			end,
			validate = function(input)
				return (not dataBaseVar[input] and strlen(input) > 0)
			end
		}
	end
end	


function Avion_Buffs:BuildEditDeBuffOptionTable(dataBaseVar, itsName)
	return {
		type = 'group',
		name = itsName,
		desc = L["Edit_Desc"],
		args = {
			enabled = {
				order = 1,
				type = 'toggle',
				name = L["Enabled"],
				desc = L["Enabled_Desc"],
				get = function()
					return dataBaseVar[itsName].enabled
				end,
				set = function(toggle)
					dataBaseVar[itsName].enabled = toggle
					
					self:UpdateEventRegistration(false)
				end,
			},
			list = {
				order = 2,
				type = 'group',
				name = L["AnimationList"],
				desc = L["AnimationList_Desc"],
				args = Avion_Core:BuildAnimationList(dataBaseVar[itsName].animationIDList),
			},
			delete = {
				order = 3,
				type = 'group',
				name = L["DeleteEntry"],
				desc = L["DeleteEntry_Desc"],
				args = {
					delete = {
						order = 1,
						type = 'execute',
						name = L["DeleteEntry"],
						desc = L["DeleteEntry_Desc"],
						func = function()
							self:DeleteDeBuff(dataBaseVar, itsName)
						end
					},
				},
			},
		},
	}
end


function Avion_Buffs:BuildEditDeBuffOptionTable_Special(dataBaseVar, itsName, unitID)
	-- Since class and name work differently, add options for each sub entry
	local listArgs = {}
	for unitName, tableData in pairs(dataBaseVar[itsName]) do
		listArgs[unitName] = self:BuildEditDeBuffOptionTable(dataBaseVar[itsName], unitName)
	end
	
	-- ... and but add a new, delete and edit option
	return {
		type = 'group',
		name = itsName,
		desc = L["Edit_Desc"],
		args = {
			new = self:BuildNewDeBuffOptionTable_Special(dataBaseVar[itsName], unitID),
			list = {
				order = 2,
				type = 'group',
				name = L["List"],
				desc = L["List_Desc"],
				args = listArgs,
			},
			delete = {
				order = 3,
				type = 'group',
				name = L["DeleteEntry"],
				desc = L["DeleteEntry_Desc"],
				args = {
					delete = {
						order = 1,
						type = 'execute',
						name = L["DeleteEntry"],
						desc = L["DeleteEntry_Desc"],
						func = function()
							self:DeleteDeBuff(dataBaseVar, itsName)
						end
					},
				},
			},
		},
	}
end


-- Build Optionstable
function Avion_Buffs:BuildOptionTable()
	local dataTypes = self.db.profile
	local dataTypeOpt, unitIDOpt, nameOpt
	
	-- Query Types (BuffGained, BuffLost, DebuffGained, DebuffLost, DebuffTypeGained)
	dataTypeOpt = {}
	for dataType, unitIDs in pairs(dataTypes) do
		-- Query userIDs (Player, Raid, Focus, Target, Name, Class)
		unitIDOpt = {}
		for unitID, dataBaseVar in pairs(unitIDs) do
			-- For each one, iterate through all (De-)Buff names and add "edit" menu
			nameOpt = {}
			for itsName, _ in pairs(dataBaseVar) do 
				-- For class and name we need to add submenus
				if (unitID == "class" or unitID == "name") then
					nameOpt[itsName] = self:BuildEditDeBuffOptionTable_Special(dataBaseVar, itsName, unitID)
				else
					nameOpt[itsName] = self:BuildEditDeBuffOptionTable(dataBaseVar, itsName)
				end
			end
			
			-- Each one has a "add new" option
			unitIDOpt[unitID] = {
				type = 'group',
				name = L[unitID],
				desc = L[unitID.."_Desc"],
				args = {
					new = self:BuildNewDeBuffOptionTable(dataBaseVar, dataType, unitID),
					edit = {
						type = 'group',
						order = 2,
						name = L["Edit"],
						desc = L["Edit_Desc"],
						args = nameOpt,
					},
				},
			}
		end
		
		-- Group those together
		dataTypeOpt[dataType] = {
			type = 'group',
			name = L[dataType],
			desc = L[dataType.."_Desc"],
			args = unitIDOpt,
		}
	end
		
	return {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = dataTypeOpt,
	}
end


-- Save and return
function Avion_Buffs:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Raid gained buff
function Avion_Buffs:SpecialEvents_PlayerBuffGained(buffName, buffIndex, applications, texture, rank, index)
	self:Trigger_PlayerEvents(self.db.profile.BuffGained.player, buffName, "PlayerBuffGained")
end


-- Raid lost buff
function Avion_Buffs:SpecialEvents_PlayerBuffLost(buffName, applications, texture, rank)
	self:Trigger_PlayerEvents(self.db.profile.BuffLost.player, buffName, "PlayerBuffLost")
end


-- Raid lost buff
function Avion_Buffs:SpecialEvents_PlayerDebuffLost(debuffName, applications, debuffType, texture, rank)
	self:Trigger_PlayerEvents(self.db.profile.DebuffLost.player, debuffName, "PlayerDebuffLost")
end


-- Raid gained a debuff
function Avion_Buffs:SpecialEvents_PlayerDebuffGained(debuffName, applications, debuffType, texture, rank, index)
	if (not self:Trigger_PlayerEvents(self.db.profile.DebuffGained.player, debuffName, "PlayerDebuffGained")) then
		self:Trigger_PlayerEvents(self.db.profile.DebuffTypeGained.player, debuffName, "PlayerDebuffTypeGained")
	end
end


-- Logic when PlayerEvent was fired
function Avion_Buffs:Trigger_PlayerEvents(dataTable, itsName, triggerID)
	if (dataTable[itsName] and dataTable[itsName].enabled) then	
		Avion_Core:TriggerAnimationsByID(triggerID..itsName, dataTable[itsName].animationIDList)
		
		return true
	else
		return false
	end
end


-- Raidmember/target gained buff
function Avion_Buffs:SpecialEvents_UnitBuffGained(unitID, buffName, index, applications, texture, rank)
	if (unitID == player) then
		return
	end
	
	self:Trigger_UnitEvents(self.db.profile.BuffGained, buffName, unitID, "UnitBuffGained")
end


-- Raidmember/target lost buff
function Avion_Buffs:SpecialEvents_UnitBuffLost(unitID, buffName, applications, texture, rank)
	if (unitID == player) then
		return
	end
	
	self:Trigger_UnitEvents(self.db.profile.BuffLost, buffName, unitID, "UnitBuffLost")
end


-- Raidmember/target lost buff
function Avion_Buffs:SpecialEvents_UnitDebuffLost(unitID, debuffName, applications, debuffType, texture, rank)
	if (unitID == player) then
		return
	end
	
	self:Trigger_UnitEvents(self.db.profile.DebuffLost, debuffName, unitID, "UnitDebuffLost")
end


-- Raidmember/target gained a debuff
function Avion_Buffs:SpecialEvents_UnitDebuffGained(unitID, debuffName, applications, debuffType, texture, rank, index)
	if (unitID == player) then
		return
	end
	
	-- Only trigger DebuffType, when Debuff by name didn't trigger
	if (not self:Trigger_UnitEvents(self.db.profile.DebuffGained, debuffName, unitID, "UnitDebuffTypeGained")) then
		self:Trigger_UnitEvents(self.db.profile.DebuffTypeGained, debuffName, unitID, "UnitDebuffTypeGained")
	end
end


-- Logic when UnitEvent was fired
function Avion_Buffs:Trigger_UnitEvents(dataTable, itsName, unitID, triggerID)
	local animationIDList

	-- Name > Focus > Target > Class > Raid
	if (dataTable.name[itsName] and dataTable.name[itsName][UnitName(unitID)] and dataTable.name[itsName][UnitName(unitID)].enabled) then
		animationIDList = dataTable.name[itsName][UnitName(unitID)].animationIDList
		
	elseif (dataTable.focus[itsName] and dataTable.focus[itsName].enabled) then
		if (unitID == "focus") then
			animationIDList = dataTable.focus[itsName].animationIDList
		end
		
	elseif (dataTable.target[itsName] and dataTable.target[itsName].enabled) then
		if (unitID == "target") then
			animationIDList = dataTable.target[itsName].animationIDList
		end
		
	elseif (dataTable.class[itsName] and dataTable.class[itsName][UnitClass(unitID)] and dataTable.class[itsName][UnitClass(unitID)].enabled) then
		animationIDList = dataTable.name[itsName][UnitName(unitID)].animationIDList
		
	elseif (dataTable.raid[itsName] and dataTable.raid[itsName].enabled) then
		if (UnitInRaid("player") and string.sub(unitID, 1, 4) == "raid" or not UnitInRaid("player") and string.sub(unitID, 1, 4) == "party") then
			animationIDList = dataTable.raid[itsName].animationIDList
		end
		
	end
		
	-- Trigger selected animation
	if (animationIDList) then
		Avion_Core:TriggerAnimationsByID(triggerID..unitID..itsName, animationIDList)
		
		return true
	else
		return false
	end
end
