-- Create new Module
local Avion_Cooldowns 	= Avion_Core:NewModule("Avion_Cooldowns")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Cooldowns")


-- Module information, default options and name
Avion_Cooldowns.name 	= "Avion_Cooldowns"

local slotNames = { 
	["HeadSlot"] = L["Head"], 
	["NeckSlot"] = L["Neck"], 
	["ShoulderSlot"] = L["Shoulder"], 
	["BackSlot"] = L["Back"], 
	["ChestSlot"] = L["Chest"], 
	["ShirtSlot"] = L["Shirt"], 
	["TabardSlot"] = L["Tabard"], 
	["WristSlot"] = L["Wrist"], 
	["HandsSlot"] = L["Hands"], 
	["WaistSlot"] = L["Waist"], 
	["LegsSlot"] = L["Legs"], 
	["FeetSlot"] = L["Feet"], 
	["Finger0Slot"] = L["Finger 1"],
	["Finger1Slot"] = L["Finger 2"],  
	["Trinket0Slot"] = L["Trinket 1"], 
	["Trinket1Slot"] = L["Trinket 2"], 
	["MainHandSlot"] = L["Mainhand"], 
	["SecondaryHandSlot"] = L["Offhand"], 
	["RangedSlot"] = L["Ranged"], 
	["AmmoSlot"] = L["Ammo"],
} 

local itemsByName = {}


-- Register SavedVariables
function Avion_Cooldowns:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Cooldowns", "profile", { 
		itemsBySlot = {
		},
		itemsByName = {
		},
		spellsByName = {
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Cooldowns")
	
	-- Make checking of item cooldowns by item name easier
	for _, groupData in pairs(self.db.profile.itemsByName) do
		for itemName, itemData in pairs(groupData.members) do
			itemsByName[itemName] = itemData
		end
	end
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Cooldowns:UpdateEventRegistration(unLoad)
	local isEnabled
	
	isEnabled = false
	for _, data in pairs(self.db.profile.itemsBySlot) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and isEnabled) then
		self:RegisterEvent("LibCD_EquippedItemCooldownEnd")
	else
		if (self:IsEventRegistered("LibCD_EquippedItemCooldownEnd")) then
			self:UnregisterEvent("LibCD_EquippedItemCooldownEnd")
		end
	end
	
	isEnabled = false
	for _, data in pairs(self.db.profile.spellsByName) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
	
	if (not unLoad and isEnabled) then
		self:RegisterEvent("LibCD_SpellCooldownEnd")
	else
		if (self:IsEventRegistered("LibCD_ItemCooldownEnd")) then
			self:UnregisterEvent("LibCD_SpellCooldownEnd")
		end
	end
	
	isEnabled = false
	for _, dataEnabled in pairs(itemsByName) do
		if (dataEnabled) then
			isEnabled = true
			break
		end
	end
	
	if (not unLoad and isEnabled) then
		self:RegisterEvent("LibCD_ItemCooldownEnd")
	else
		if (self:IsEventRegistered("LibCD_ItemCooldownEnd")) then
			self:UnregisterEvent("LibCD_ItemCooldownEnd")
		end
	end
end


-- Build Optionstable
function Avion_Cooldowns:BuildOptionTable()
	-- Suboptions for spells by name
	local spellsByNameOpt = {}
	for spellName, data in pairs(self.db.profile.spellsByName) do
		spellsByNameOpt[spellName] = {
			type = 'group',
			name = spellName,
			desc = L["SpellName_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
				delete = {
					order = 3,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.spellsByName[spellName] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end
	
	local itemsByNameOpt = {}
	local itemsByNameSubOpt
	for groupName, groupData in pairs(self.db.profile.itemsByName) do
		itemsByNameSubOpt = {}
		for itemName, itemData in pairs(groupData.members) do
			itemsByNameSubOpt[itemName] = {
				type = 'group',
				name = itemName,
				desc = L["SlotName_Desc"],
				args = { 
					enabled = {
						order = 1,
						type = 'toggle',
						name = L["Enabled"],
						desc = L["Enabled_Desc"],
						get = function()
							return itemData	-- !!! Testen
						end,
						set = function(toggle)
							itemData = toggle	-- !!! Testen
							itemsByName[itemName] = toggle
							
							self:UpdateEventRegistration(false)
						end,
					},
					delete = {
						order = 2,
						type = 'group',
						name = L["DeleteEntry"],
						desc = L["DeleteEntry_Desc"],
						args = {
							delete = {
								order = 1,
								type = 'execute',
								name = L["DeleteEntry"],
								desc = L["DeleteEntry_Desc"],
								func = function()
									-- Delete from DB
									self.db.profile.itemsByName[groupName].members[itemName] = nil
									itemsByName[itemName] = nil
									
									-- Update option table
									Avion_Core:UpdateModuleOptionTable(self)
									
									-- Update events
									self:UpdateEventRegistration(false)
								end
							},
						},
					},
				},
			}
		end
		
		itemsByNameOpt[groupName] = {
			type = 'group',
			name = groupName,
			desc = L["SlotName_Desc"],
			args = { 
				new = {
					type = 'text',
					order = 1,
					name = L["New_ItemByName"],
					desc = L["New_ItemByName_Desc"],
					usage = L["New_ItemByName_Usage"],
					get = function()
						return ""
					end,
					set = function(input)
						-- Test input correctness
						if (self.db.profile.itemsByName[groupName].members[input] ~= nil or strlen(input) <= 0) then
							return
						end
						
						-- Insert into DB
						self.db.profile.itemsByName[groupName].members[input] = true
						
						-- Update option table
						Avion_Core:UpdateModuleOptionTable(self)
						
						-- Update events
						self:UpdateEventRegistration(false)
					end,
				},
				items = {
					type = 'group',
					order = 2,
					name = L["ItemByName"],
					desc = L["ItemByName_Desc"],
					args = itemsByNameSubOpt,
				},
				list = {
					order = 3,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(groupData.animationIDList),
				},
				delete = {
					order = 4,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.itemsByName[groupName] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end
	
	local itemsBySlotOpt = {}
	for slotName, data in pairs(self.db.profile.itemsBySlot) do
		itemsBySlotOpt[slotName] = {
			type = 'group',
			name = slotNames[slotName],
			desc = L["SlotName_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
				delete = {
					order = 3,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.itemsBySlot[slotName] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end
	
	-- Build list o slotnames
	local slotNameOpt = {}
	for slotName, LslotName in pairs(slotNames) do
		if (not self.db.profile.itemsBySlot[slotName]) then
			slotNameOpt[slotName] = {
				type = 'execute',
				order = 3,
				name = LslotName,
				desc = L["New_ItemBySlot_Desc"],
				get = function()
					return ""
				end,
				func = function()
					-- Test input correctness
					if (self.db.profile.itemsBySlot[slotName]) then
						return
					end
					
					-- Insert into DB
					self.db.profile.itemsBySlot[slotName] = {
						animationIDList = { },
						enabled = true,
					}
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end,
			}
		end
	end

	-- AceOptionTable
	return {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'group',
				order = 1,
				name = L["NewCore"],
				desc = L["NewCore_Desc"],
				args = {
					spellsByName = {
						type = 'text',
						order = 1,
						name = L["New_SpellByName"],
						desc = L["New_SpellByName_Desc"],
						usage = L["New_SpellByName_Usage"],
						get = function()
					        return ""
					    end,
					    set = function(input)
					        -- Test input correctness
							if (self.db.profile.spellsByName[input] or strlen(input) <= 0) then
								return
							end
							
					        -- Insert into DB
							self.db.profile.spellsByName[input] = {
								animationIDList = { },
								enabled = true,
							}
							
							-- Update option table
							Avion_Core:UpdateModuleOptionTable(self)
							
							-- Update events
							self:UpdateEventRegistration(false)
					    end,
					},
					itemsByName = {
						type = 'text',
						order = 2,
						name = L["New_Group_ItemByName"],
						desc = L["New_Group_ItemByName_Desc"],
						usage = L["New_Group_ItemByName_Usage"],
						get = function()
					        return ""
					    end,
					    set = function(input)
					       -- Test input correctness
							if (self.db.profile.itemsByName[input] or strlen(input) <= 0) then
								return
							end
							
					        -- Insert into DB
							self.db.profile.itemsByName[input] = {
								animationIDList = { },
								members = {},
							}
							
							-- Update option table
							Avion_Core:UpdateModuleOptionTable(self)
							
							-- Update events
							self:UpdateEventRegistration(false)
					    end,
					},
					itemsBySlot = {
						type = 'group',
						order = 3,
						name = L["New_ItemBySlot"],
						desc = L["New_ItemBySlot_Desc"],
						args = slotNameOpt,
					},
				},
			},
			spellsByName = {
				type = 'group',
				order = 2,
				name = L["Edit_SpellByName"],
				desc = L["Edit_SpellByName_Desc"],
				args = spellsByNameOpt,
			},
			itemsByName = {
				type = 'group',
				order = 3,
				name = L["Edit_ItemByName"],
				desc = L["Edit_ItemByName_Desc"],
				args = itemsByNameOpt,
			},
			itemsBySlot = {
				type = 'group',
				order = 4,
				name = L["Edit_ItemBySlot"],
				desc = L["Edit_ItemBySlot_Desc"],
				args = itemsBySlotOpt,
			},
		},
	}
end


-- Save and return
function Avion_Cooldowns:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Cooldowns:AnimationDeleted(animationID)
	for _, data in pairs(self.db.profile.spellsByName) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	for _, data in pairs(self.db.profile.itemyByName) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	for _, data in pairs(self.db.profile.itemsBySlot) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
end


-- Spell cooldown ended
function Avion_Cooldowns:LibCD_SpellCooldownEnd(spellName, spellID, startTime)
	if (self.db.profile.spellsByName[spellName] and self.db.profile.spellsByName[spellName].enabled) then
		Avion_Core:TriggerAnimationsByID("SpellCooldown_" .. spellName, self.db.profile.spellsByName[spellName].animationIDList)
	end
end


-- Item cooldown ended
function Avion_Cooldowns:LibCD_ItemCooldownEnd(itemName, itemID, startTime, bagID, slotID)
	-- Local if enabled in list
	if (itemsByName[itemName]) then
		-- Search for item in all groups
		for groupName, groupData in pairs(self.db.profile.itemsByName) do
			-- Found  in this group?
			if (groupData.members[itemName]) then
				Avion_Core:TriggerAnimationsByID("ItemCooldownByName_" .. itemName, groupData.animationIDList)
				
				-- Can only be added once!
				break
			end
		end
	end
end


-- Cooldown of equipped item ended
function Avion_Cooldowns:LibCD_EquippedItemCooldownEnd(itemName, itemID, startTime, slotID)
	if (self.db.profile.itemsBySlot[slotID] and self.db.profile.itemsBySlot[slotID].enabled) then
		if (not itemsByName[itemName]) then
			Avion_Core:TriggerAnimationsByID("ItemCooldownBySlot_" .. slotID, self.db.profile.itemsBySlot[slotID].animationIDList)
		end
	end
end
