-- Create new Module
local Avion_Mana 	= Avion_Core:NewModule("Avion_Mana")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Mana")


-- Module information, default options and name
Avion_Mana.name 	= "Avion_Mana"


-- Register SavedVariables
function Avion_Mana:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Mana", "profile", { 
		threshold = {
		},
		tick = {
			animationIDList = { },
			enabled = false,
		}
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Mana")
	
	self.lastPercentage = UnitMana("player")
	self.lastPowerType = UnitPowerType("player")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Mana:UpdateEventRegistration(unLoad)
	local isEnabled
	
	isEnabled = false
	for _, data in pairs(self.db.profile.threshold) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and (isEnabled or self.db.profile.tick.enabled)) then
		self:RegisterEvent("UNIT_MANA")
	else
		if (self:IsEventRegistered("UNIT_MANA")) then
			self:UnregisterEvent("UNIT_MANA")
		end
	end
end


-- Build Optionstable
function Avion_Mana:BuildOptionTable()
	local thresholdOpt = {}
	for index, data in pairs(self.db.profile.threshold) do
		thresholdOpt[index] = {
			type = 'group',
			name = L["Threshold"] .. index,
			desc = L["Threshold_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
				delete = {
					order = 3,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.threshold[index] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end

	-- AceOptionTable
	return {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'text',
				order = 1,
				name = L["New"],
				desc = L["New_Desc"],
				usage = L["New_Usage"],
				get = function()
					return ""
				end,
				set = function(input)
					local no = tonumber(input)
					
					if (self.db.profile.threshold[no] or strlen(input) == 0) then
						return
					end
					
					-- Insert into DB
					self.db.profile.threshold[no] = {
						animationIDList = { },
						enabled = true,
					}
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end,
			},
			edit = {
				type = 'group',
				order = 2,
				name = L["Edit"],
				desc = L["Edit_Desc"],
				args = thresholdOpt,
			},
			tick = {
				type = 'group',
				order = 3,
				name = L["Tick"],
				desc = L["Tick_Desc"],
				args = {
					enabled = {
						order = 1,
						type = 'toggle',
						name = L["Enabled"],
						desc = L["Enabled_Desc"],
						get = function()
							return self.db.profile.tick.enabled
						end,
						set = function(toggle)
							self.db.profile.tick.enabled = toggle
							
							self:UpdateEventRegistration(false)
						end,
					},
					list = {
						order = 2,
						type = 'group',
						name = L["AnimationList"],
						desc = L["AnimationList_Desc"],
						args = Avion_Core:BuildAnimationList(self.db.profile.tick.animationIDList),
					},
				},
			},
		},
	}
end


-- Save and return
function Avion_Mana:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Mana:AnimationDeleted(animationID)
	for _, data in pairs(self.db.profile.threshold) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	if (self.db.profile.tick.animationIDList[animationID]) then
		self.db.profile.tick.animationIDList[animationID] = nil
	end
end


-- Fire or update low health event
function Avion_Mana:UNIT_MANA(unitID)
	-- But only for player
	if (unitID ~= "player") then
		return
	end
	
	-- Save current mana
	local currentPercentage = UnitMana("player") / UnitManaMax("player") * 100
	local currentPowerType = UnitPowerType("player")
	
	-- Powertype changed? (eg. shapeshifted) If so, update self.lastPercentage!
	if (currentPowerType ~= self.lastPowerType) then
		self.lastPowerType = currentPowerType
		
		-- We changed back from rage to mana, if mana not full, start 5-seconrd rule!
		if (currentPercentage < 100) then
			self.lastPercentage = currentPercentage + 0.1
		else
			self.lastPercentage = currentPercentage
		end
	end

	-- Check if mana is growing or shrinking
	if (currentPercentage < self.lastPercentage) then
		-- Mana tick (since mana went down, we SHOULD have cast something or at least 5 second rule should be started)
		if (self.db.profile.tick.enabled) then
			self:ScheduleEvent(
				"Avion_ManaTick_Shedule", 
				Avion_Core.TriggerAnimationsByID, 
				5, 
				Avion_Core, 
				"ManaTick",
				self.db.profile.tick.animationIDList
			)
		end
		
		-- Test each treshhold
		for threshold, data in pairs(self.db.profile.threshold) do
			-- Check if health is inside interval [newHealth - oldHealth)
			if (data.enabled and currentPercentage <= threshold and threshold < self.lastPercentage) then
				Avion_Core:TriggerAnimationsByID("ManaThreshold", data.animationIDList)
			end
		end
	end
	
	-- Update last mana
	self.lastPercentage = currentPercentage
end
