-- Create new Module
local Avion_Rage 	= Avion_Core:NewModule("Avion_Rage")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Rage")


-- Module information, default options and name
Avion_Rage.name 	= "Avion_Rage"


-- Register SavedVariables
function Avion_Rage:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Rage", "profile", { 
		threshold = {
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Rage")
	
	self.lastPercentage = UnitMana("player")
	self.lastPowerType = UnitPowerType("player")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Rage:UpdateEventRegistration(unLoad)
	local isEnabled
	
	isEnabled = false
	for _, data in pairs(self.db.profile.threshold) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and isEnabled) then
		self:RegisterEvent("UNIT_RAGE")
	else
		if (self:IsEventRegistered("UNIT_RAGE")) then
			self:UnregisterEvent("UNIT_RAGE")
		end
	end
end


-- Build Optionstable
function Avion_Rage:BuildOptionTable()
	local thresholdOpt = {}
	for index, data in pairs(self.db.profile.threshold) do
		thresholdOpt[index] = {
			type = 'group',
			name = L["Threshold"] .. index,
			desc = L["Threshold_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
				delete = {
					order = 3,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.threshold[index] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end

	-- AceOptionTable
	return {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'text',
				order = 1,
				name = L["New"],
				desc = L["New_Desc"],
				usage = L["New_Usage"],
				get = function()
					return ""
				end,
				set = function(input)
					local no = tonumber(input)
					
					if (self.db.profile.threshold[no] or strlen(input) == 0) then
						return
					end
					
					-- Insert into DB
					self.db.profile.threshold[no] = {
						animationIDList = { },
						enabled = true,
					}
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end,
			},
			edit = {
				type = 'group',
				order = 2,
				name = L["Edit"],
				desc = L["Edit_Desc"],
				args = thresholdOpt,
			},
		},
	}
end


-- Save and return
function Avion_Rage:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Rage:AnimationDeleted(animationID)
	for _, data in pairs(self.db.profile.threshold) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
end


-- Fire or update low health event
function Avion_Rage:UNIT_RAGE(unitID)	
	-- But only for player
	if (unitID ~= "player") then
		return
	end
	
	-- Save current mana
	local currentPercentage = UnitMana("player") / UnitManaMax("player") * 100
	local currentPowerType = UnitPowerType("player")
	
	-- Powertype changed? (eg. shapeshifted) If so, update self.lastPercentage!
	if (currentPowerType ~= self.lastPowerType) then
		self.lastPowerType = currentPowerType
		self.lastPercentage = currentPercentage
	end

	-- Check if rage is growing or shrinking
	if (self.lastPercentage < currentPercentage) then
		-- Test each treshhold
		for threshold, eventIDs in pairs(self.db.profile.triggers.RageThreshold.data) do
			-- Check if rage is inside interval [newRage - oldRage)
			if (data.enabled and self.lastPercentage < threshold and threshold <= currentPercentage) then
				Avion_Core:TriggerAnimationsByID("RageThreshold", data.animationIDList)
			end
		end
	end
	
	-- Update current/last mana/rage
	self.lastPercentage = currentPercentage
end
